###########################################
## Figures shown in the main manuscript. ##
###########################################

## Required packages.
library("bamlss")
library("gamlss")
library("zoo")
library("automap")
library("spdep")
library("raster")
library("rworldmap")

## Help functions for creating the figures.
save_img <- TRUE

simg <- function(file, height = 4, width = 5, type = "png", res = 250) {
  if(save_img) {
    file <- paste(file, type, sep = ".")
    if(type == "png") {
      png(file, height = height, width = width,
        units = "in", res = res, bg = "transparent")
    }
    if(tolower(type) == "jpg") {
      jpeg(file, height = height, width = width,
        units = "in", res = res, bg = "white")
    }
    if(type == "pdf") {
      cairo_pdf(file, height = height, width = width)
    }
    if(type == "ps") {
      cairo_pdf(file, height = height, width = width)
    }
  } else x11(width = width, height = height)
}

## Function to create plots of the maximum ACF of all samples. 
plot_max_acf <- function(x, chain = NULL, ...) {
  if(!inherits(x, "mcmc.list"))
    x <- as.mcmc.list(list(x))
  if(!is.null(chain))
    x <- x[chain]
  macf <- list()
  for(j in seq_along(x)) {
    macf[[j]] <- if(is.null(dim(x[[j]]))) {
      list(acf(x[[j]], plot = FALSE, ...))
    } else {
      apply(x[[j]], 2, function(x) { acf(x, plot = FALSE, na.action = na.omit, ...) })
    }
  }
  acfx <- macf[[1]][[1]]
  macf <- lapply(macf, function(x) { lapply(x, function(z) { z$acf }) })
  for(j in seq_along(macf))
    macf[[j]] <- apply(do.call("rbind", macf[[j]]), 2, max, na.rm = TRUE)
  macf <- apply(do.call("rbind", macf), 2, max)
  acfx$acf <- array(macf, dim = c(length(acfx$acf), 1L, 1L))
  args <- list(...)
  if(is.null(args$main))
    args$main <- "Maximum ACF of samples"
  if(is.null(args$xlab))
    args$xlab <- "Lag"
  if(is.null(args$ylab))
    args$ylab <- "ACF"
  getS3method("plot", class = "acf")(acfx, main = args$main, xlab = args$xlab,
    ylab = args$ylab, xlim = args$xlim, ylim = args$ylim)
}

## Load the temperature data.
load("Tdata.rda")

## Compute lag-difference variable.
Tdata$T_diff <- Tdata$T - Tdata$T_lag

## Empty figures folder.
if(!file.exists("figures"))
  dir.create("figures")

## Data plots.
simg("figures/Tdata-hist", height = 3, width = 4.25)
par(mar = c(4.1, 4.1, 1.1, 1.1))
hist(Tdata$T, breaks = 50, xlab = "Temperature", main = "",
  col = "lightgray", freq = FALSE)
lines(density(Tdata$T))
box()
Tr <- range(Tdata$T, na.rm = TRUE)
Tm <- mean(Tdata$T, na.rm = TRUE)
legend("topleft", c(expression(paste(T[min], " = ", -33.1)), expression(paste(T[max], " =  ", 13.3)),
  expression(paste(T[ave], " =  ", -4.6))), box.col = NA, bg = NA)
abline(v = Tm, lty = 2, lwd = 1.5, cex = 0.8)
dev.off()

blue <- rgb(0, 0.4, 0.8)

simg("figures/Tdata-vs-yday", height = 3, width = 4.25)
par(mar = c(4.1, 4.1, 1.1, 1.1))
plot(T ~ yday, data = Tdata, pch = ".", xlab = "Time", ylab = "Temperature",
  axes = FALSE, type = "n", main = "")
k <- 1
for(i in unique(Tdata$year)) {
  d <- subset(Tdata, year == i)
  d <- d[order(d$yday), ]
  lines(d$T ~ d$yday, col = rgb(0.1, 0.1, 0.1, alpha = 0.05))
  k <- k + 1
}
mean_yday <- q5 <- q95 <- NULL
for(i in 1:365) {
  mean_yday <- c(mean_yday, mean(subset(Tdata, yday == i)$T, na.rm = TRUE))
  q5 <- c(q5, quantile(subset(Tdata, yday == i)$T, probs = 0.05, na.rm = TRUE))
  q95 <- c(q95, quantile(subset(Tdata, yday == i)$T, probs = 0.95, na.rm = TRUE))
}
lines(1:365, mean_yday, col = "white", lwd = 2)
lines(1:365, q5, col = "lightgray", lwd = 2.5, lty = 1)
lines(1:365, q95, col = "lightgray", lwd = 2.5, lty = 1)

lines(1:365, mean_yday, col = rainbow_hcl(1), lwd = 0.8)
lines(1:365, q5, col = blue, lwd = 1, lty = 1)
lines(1:365, q95, col = blue, lwd = 1, lty = 1)
Axis(as.Date(c("1970-01-01", "1970-12-31")), side = 1)
axis(2)
box()
legend("bottom", c("Raw data", "Daily averages", "5% and 95% daily quantiles"), lwd = c(1, 1, 1),
  col = c("black", rainbow_hcl(1), blue), lty = c(1, 1, 1),
  box.col = NA, bg = NA, cex = 0.7)
dev.off()

simg("figures/Tdata-series", height = 6, width = 8.5)
par(mar = c(4.1, 4.1, 1.1, 1.1))
T <- na.approx(zoo(Tdata$T,
  as.Date(paste(Tdata$year, "-", Tdata$month, "-", Tdata$day, sep = ""), format = "%Y-%m-%d")))
plot(T, col = "black", xlab = "Time", ylab = "Temperature")
abline(h = mean(T), lty = 2, col = "lightgray")
ymean <- NULL
for(i in sort(unique(Tdata$year)))
  ymean <- c(ymean, mean(subset(Tdata, year == i)$T, na.rm = TRUE))
ymean <- zoo(ymean, as.Date(paste(sort(unique(Tdata$year)), "-06-01", sep = "")))
rmean <- rollapply(T, FUN = mean, width = 365*20, align = "right")
r5 <- rollapply(T, FUN = quantile, probs = 0.025, width = 365*20, align = "right")
r95 <- rollapply(T, FUN = quantile, probs = 0.975, width = 365*20, align = "right")
lines(rmean, col = "white", lwd = 2)
lines(r5, col = "lightgray", lwd = 3.5)
lines(r95, col = "lightgray", lwd = 3.5)
lines(rmean, col = rainbow_hcl(2), lwd = 0.8)
lines(r5, col = blue, lwd = 1)
lines(r95, col = blue, lwd = 1)
legend("bottomright", c("Raw data", "20y run. mean", "20y run. quant., 2.5% and 97.5%"), lwd = 1.5,
  col = c("black", rainbow_hcl(1), blue), lty = 1,
  box.col = NA, bg = NA, horiz = TRUE, cex = 0.7)
dev.off()

## Load the temperature models.
load("Tmodel-b0.rda")
load("Tmodel-b1.rda")
load("Tmodel-b2.rda")

## Extract samples for creating traceplots.
i <- "mu.s.ti(yday,year).tau21"
s0 <- samples(b0, coef.only = TRUE)
s1 <- samples(b1, coef.only = TRUE)
s2 <- samples(b2, coef.only = TRUE, combine = FALSE)
s3 <- as.mcmc.list(lapply(s2, function(x) { x[, i, drop = FALSE] }))

simg("figures/traceplot-01", height = 3, width = 8.5)
par(mfrow = c(1, 3), mar = c(4.1, 4.1, 4.1, 1.1))
plot(s0[, i], density = FALSE, main = "Coefficient sampling path", 
  auto.layout = FALSE)
lines(lowess(as.vector(time(s0[, i])), as.numeric(s0[, i])), col = rainbow_hcl(1), lwd = 1)
acf(s0[, i], main = "Empirical ACF", lag.max = 100)
plot_max_acf(s0, lag.max = 100)
dev.off()

simg("figures/traceplot-02", height = 3, width = 8.5)
par(mfrow = c(1, 3), mar = c(4.1, 4.1, 4.1, 1.1))
plot(s1[, i], density = FALSE, main = "Coefficient sampling path", 
  auto.layout = FALSE)
lines(lowess(as.vector(time(s1[, i])), as.numeric(s1[, i])), col = rainbow_hcl(1), lwd = 1)
acf(s1[, i], main = "Empirical ACF", lag.max = 100)
plot_max_acf(s1, lag.max = 100)
dev.off()

simg("figures/traceplot-03", height = 3, width = 8.5)
par(mfrow = c(1, 3), mar = c(4.1, 4.1, 4.1, 1.1))
cols <- heat_hcl(8, h = c(255, 230))
plot(s3, density = FALSE, main = "Coefficient sampling path", 
  auto.layout = FALSE, col = cols, smooth = FALSE, lty = 1, ylim = range(s3) + c(0, 7))
for(i in 1:8)
  lines(lowess(as.vector(time(s3[[i]])), as.numeric(s3[[i]])), col = cols[i], lwd = 1)
legend("topleft", paste("Chain", 1:8), lwd = 1, col = cols, box.col = NA, bg = NA, cex = 0.9, ncol = 2)
acf(bamlss:::process.chains(s3, drop = TRUE), main = "Empirical ACF", lag.max = 100)
plot_max_acf(samples(b2, coef.only = TRUE, combine = TRUE), lag.max = 100, chain = 1)
dev.off()

## Load the BCPE model b4.
load("Tmodel-b4.rda")

## Predict time-varying seasonal effects.
fseason <- list(
  "mu" = matrix(0, 365, length(unique(Tdata$year))),
  "sigma" = matrix(0, 365, length(unique(Tdata$year))),
  "nu" = matrix(0, 365, length(unique(Tdata$year)))
)

for(i in seq_along(unique(Tdata$year))) {
  cat("year", unique(Tdata$year)[i], "\n")
  nd <- data.frame("year" = unique(Tdata$year)[i], "yday" = 1:365)
  fseason$mu[, i] <- predict(b4, newdata = nd, model = "mu", term = "yday", intercept = FALSE)
  fseason$sigma[, i] <- predict(b4, newdata = nd, model = "sigma", term = "yday", intercept = FALSE)
  fseason$nu[, i] <- predict(b4, newdata = nd, model = "nu", term = "yday", intercept = FALSE)
}

## Predict the main effects.
fm_mu <- predict(b4, newdata = data.frame(yday = 1:365), model = "mu", term = "ti(yday)", intercept = FALSE)
fm_sigma <- predict(b4, newdata = data.frame(yday = 1:365), model = "sigma", term = "ti(yday)", intercept = FALSE)
fm_nu <- predict(b4, newdata = data.frame(yday = 1:365), model = "nu", term = "ti(yday)", intercept = FALSE)

## Plot model b4, seasonal effects.
simg("figures/b4-season-mu", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
redblue1 <- hcl(c(260, 0, 0), c(80, 0, 80), 30, alpha = c(0.15, 0.1, 0.15))
redblue2 <- hcl(c(260, 0, 0), c(80, 0, 80), 30, alpha = 1)
colbreaks <- seq.int(min(Tdata$year), max(Tdata$year), length = 4)
ycat <- cut(unique(Tdata$year), breaks = colbreaks, include.lowest = TRUE)
matplot(1:365, fseason$mu, type = "l", col = redblue1[ycat], lty = 1, axes = FALSE,
  xlab = "Time", ylab = expression(paste("Effects on ", log(mu))), cex = 0.5)
lines(fm_mu ~ c(1:365), lty = 2, lwd = 2)
abline(h = 0, lty = 2, col = "lightgray")
Axis(as.Date(c("1970-01-01", "1970-12-31")), side = 1)
axis(2)
box()
legend("bottom", c("1900-1939", "1940-1979", "1980-2015", "Main effect"), lwd = 1.5,
  col = c(redblue2, "black"), lty = c(1, 1, 1, 2),
  box.col = NA, bg = NA, cex = 0.8)
dev.off()

simg("figures/b4-season-sigma", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
matplot(1:365, fseason$sigma, type = "l", col = redblue1[ycat], lty = 1, axes = FALSE,
  xlab = "Time", ylab = expression(paste("Effects on ", log(sigma))))
lines(fm_sigma ~ c(1:365), lty = 2, lwd = 2)
abline(h = 0, lty = 2, col = "lightgray")
Axis(as.Date(c("1970-01-01", "1970-12-31")), side = 1)
axis(2)
box()
legend("top", c("1900-1939", "1940-1979", "1980-2015", "Main effect"), lwd = 1.5,
  col = c(redblue2, "black"), lty = c(1, 1, 1, 2),
  box.col = NA, bg = NA, cex = 0.8)
dev.off()

simg("figures/b4-season-tau", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
matplot(1:365, fseason$nu, type = "l", col = redblue1[ycat], lty = 1, axes = FALSE,
  xlab = "Time", ylab = expression(paste("Effects on ", nu)))
lines(fm_nu ~ c(1:365), lty = 2, lwd = 2)
abline(h = 0, lty = 2, col = "lightgray")
Axis(as.Date(c("1970-01-01", "1970-12-31")), side = 1)
axis(2)
box()
legend("top", c("1900-1939", "1940-1979", "1980-2015", "Main effect"), lwd = 1.5,
  col = c(redblue2, "black"), lty = c(1, 1, 1, 2),
  box.col = NA, bg = NA, cex = 0.8)
dev.off()

## Model b4, effects of the time trend.
simg("figures/b4-trend-mu", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
plot(b4, model = "mu", term = "ti(year)", scheme = 2, grid = 100,
  xlab = "Time", ylab = expression(paste("Effect on ", log(mu))), spar = FALSE)
abline(h = 0, lty = 2, col = "lightgray")
dev.off()

simg("figures/b4-trend-sigma", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
plot(b4, model = "sigma", term = "ti(year)", scheme = 2, grid = 100,
  xlab = "Time", ylab = expression(paste("Effect on ", log(sigma))), spar = FALSE)
abline(h = 0, lty = 2, col = "lightgray")
dev.off()

simg("figures/b4-trend-nu", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
plot(b4, model = "nu", term = "ti(year)", scheme = 2, grid = 100,
  xlab = "Time", ylab = expression(paste("Effect on ", nu)), spar = FALSE)
abline(h = 0, lty = 2, col = "lightgray")
dev.off()

## Model b4, lag effects.
simg("figures/b4-Tdiff-mu", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
plot(b4, model = "mu", term = "s(T_diff)", scheme = 2, grid = 100,
  xlab = expression(T[t] - T[t - 1]), ylab = expression(paste("Effect on ", log(mu))), spar = FALSE)
abline(h = 0, lty = 2, col = "lightgray")
dev.off()

simg("figures/b4-Tdiff-sigma", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
plot(b4, model = "sigma", term = "s(T_diff)", scheme = 2, grid = 100,
  xlab = expression(T[t] - T[t - 1]), ylab = expression(paste("Effect on ", log(sigma))), spar = FALSE)
abline(h = 0, lty = 2, col = "lightgray")
dev.off()

simg("figures/b4-Tdiff-nu", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
plot(b4, model = "nu", term = "s(T_diff)", scheme = 2, grid = 100,
  xlab = expression(T[t] - T[t - 1]), ylab = expression(paste("Effect on ", nu)), spar = FALSE)
abline(h = 0, lty = 2, col = "lightgray")
dev.off()

## Quantile residuals.
## Model b2.
load("Tmodel-b2.rda")

simg("figures/b2-qresids-hist", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 2.1, 1.1), cex = 0.7)
plot(b2, which = 3, main = expression(N(mu, sigma)), col = "lightgray", breaks = 40,
  xlim = c(-4, 4), spar = FALSE)
dev.off()

simg("figures/b2-qresids-qqplot", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 2.1, 1.1), cex = 0.7)
res <- residuals(b2, FUN = function(x) { x }, nsamps = 1000)
plot(res, which = "qq-resid", main = expression(N(mu, sigma)), pch = ".", cex = 2, spar = FALSE)
dev.off()

## Model b3.
load("Tmodel-b3.rda")

simg("figures/b3-qresids-hist", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 2.1, 1.1), cex = 0.7)
plot(b3, which = 3, main = expression(BCT(mu, sigma, nu , tau)), col = "lightgray", breaks = 40,
  xlim = c(-4, 4), spar = FALSE)
dev.off()

simg("figures/b3-qresids-qqplot", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 2.1, 1.1), cex = 0.7)
res <- residuals(b3, FUN = function(x) { x }, nsamps = 1000)
plot(res, which = "qq-resid", main = expression(BCT(mu, sigma, nu , tau)), pch = ".", cex = 2, spar = FALSE)
dev.off()

## Model b4.
load("Tmodel-b4.rda")

simg("figures/b4-qresids-hist", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 2.1, 1.1), cex = 0.7)
plot(b4, which = 3, main = expression(BCPE(mu, sigma, nu , tau)), col = "lightgray", breaks = 40,
  xlim = c(-4, 4), spar = FALSE)
dev.off()

simg("figures/b4-qresids-qqplot", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 2.1, 1.1), cex = 0.7)
res <- residuals(b4, FUN = function(x) { x }, nsamps = 1000)
plot(res, which = "qq-resid", main = expression(BCPE(mu, sigma, nu , tau)), pch = ".", cex = 2, spar = FALSE)
dev.off()

## Time trend effect plot for parameter mu.
simg("figures/b4-temptrend-mu", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
shift <- abs(min(Tdata$T)) + 1
nd <- data.frame("year" = sort(unique(Tdata$year)))
p <- predict(b4, newdata = nd, model = "mu", term = "ti(year)", FUN = function(x) { x })
p <- exp(p) - shift
p <- t(apply(p, 1, function(x) {
  c(quantile(x, prob = 0.01/2), "Mean" = mean(x),  quantile(x, prob = 1- 0.01/2))}))
plot2d(p ~ nd$year, fill.select = c(0, 1, 0, 1), scheme = 2, grid = 200, xlab = "Time",
  ylab = "Temperature", axes = FALSE, xlim = c(1900, 2020))
axis(1, at = seq(1900, 2020, by = 20))
axis(2)
dev.off()

## Predict time trend quantiles.
shift <- abs(min(Tdata$T)) + 1
nd <- data.frame("year" = sort(unique(Tdata$year)))
p <- predict(b4, newdata = nd, term = "ti(year)", FUN = function(x) { x })

bcpe_quantiles <- function(x) {
  q2p5 <- q97p5 <- q10 <- q90 <- q50 <- 0
  for(i in 1:ncol(x$mu)) {
    q2p5 <- q2p5 + qBCPE(0.025, mu = exp(x$mu[, i]), sigma = exp(x$sigma[, i]), nu = x$nu[, i], tau = x$tau[, i])
    q97p5 <- q97p5 + qBCPE(0.975, mu = exp(x$mu[, i]), sigma = exp(x$sigma[, i]), nu = x$nu[, i], tau = x$tau[, i])
    q10 <- q10 + qBCPE(0.1, mu = exp(x$mu[, i]), sigma = exp(x$sigma[, i]), nu = x$nu[, i], tau = x$tau[, i])
    q90 <- q90 + qBCPE(0.9, mu = exp(x$mu[, i]), sigma = exp(x$sigma[, i]), nu = x$nu[, i], tau = x$tau[, i])
    q50 <- q50 + qBCPE(0.5, mu = exp(x$mu[, i]), sigma = exp(x$sigma[, i]), nu = x$nu[, i], tau = x$tau[, i])
  }
  q2p5 <- q2p5 / ncol(x$mu)
  q97p5 <- q97p5 / ncol(x$mu)
  q10 <- q10 / ncol(x$mu)
  q90 <- q90 / ncol(x$mu)
  q50 <- q50 / ncol(x$mu)
  cbind("2.5%" = q2p5, "10%" = q10, "50%" = q50, "90%" = q90, "97.5%" = q97p5)
}

fq <- bcpe_quantiles(p) - shift
fqs <- apply(fq, 2, function(x) {
  f <- splinefun(nd$year, x)
  f(nd$year, deriv = 1)
})

simg("figures/b4-temptrend-quantiles", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
T <- na.approx(zoo(Tdata$T,
  as.Date(paste(Tdata$year, "-", Tdata$month, "-", Tdata$day, sep = ""), format = "%Y-%m-%d")))
plot(T, col = "black", xlab = "Time", ylab = "Temperature")
fqz <- as.zoo(fq, as.Date(as.character(nd$year), format = "%Y"))
for(j in 1:ncol(fqz)) {
  lines(fqz[, j], col = "white", lwd = 2.5)
  lines(fqz[, j], col = blue, lwd = 1.5)
}
dev.off()

simg("figures/b4-temptrend-quantiles-slopes", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
col <- rainbow_hcl(5)
plot2d(fqs ~ nd$year, xlab = "Time", ylab = "Temperature change", lwd = 1.5,
  col.lines = col, type = "n", axes = FALSE, xlim = c(1900, 2020))
for(j in 1:ncol(fqs)) {
  lines(fqs[, j] ~ nd$year, col = "black", lwd = 3)
  lines(fqs[, j] ~ nd$year, col = col[j], lwd = 2)
}
axis(1, at = seq(1900, 2020, by = 20))
axis(2, at = c(0, 0.02, 0.04, 0.06))
abline(h = 0, lty = 2, col = "lightgray", lwd = 1.5)
legend("topleft", rev(colnames(fqs)), lwd = 2, col = rev(col), box.col = NA, bg = NA, cex = 0.8)
box()
dev.off()

## Pictures for intro, simulated data.
set.seed(123)
n <- 200
d <- data.frame("x" = runif(n))
d$y <- 1.2 + 5 * d$x + rnorm(n, sd = exp(-4.5 + 6 * d$x))

b1 <- bamlss(y ~ x, data = d)
b2 <- bamlss(list(y ~ x, sigma ~ x), data = d)

fit1 <- predict(b1, FUN = function(x) { x })
fit2 <- predict(b2, FUN = function(x) { x })

norm_quantiles <- function(x) {
  q2p5 <- q97p5 <- q10 <- q90 <- q50 <- 0
  for(i in 1:ncol(x$mu)) {
    q2p5 <- q2p5 + qnorm(0.025, mean = x$mu[, i], sd = exp(x$sigma[, i]))
    q97p5 <- q97p5 + qnorm(0.975, mean = x$mu[, i], sd = exp(x$sigma[, i]))
    q10 <- q10 + qnorm(0.1, mean = x$mu[, i], sd = exp(x$sigma[, i]))
    q90 <- q90 + qnorm(0.9, mean = x$mu[, i], sd = exp(x$sigma[, i]))
    q50 <- q50 + qnorm(0.5, mean = x$mu[, i], sd = exp(x$sigma[, i]))
  }
  q2p5 <- q2p5 / ncol(x$mu)
  q97p5 <- q97p5 / ncol(x$mu)
  q10 <- q10 / ncol(x$mu)
  q90 <- q90 / ncol(x$mu)
  q50 <- q50 / ncol(x$mu)
  cbind("2.5%" = q2p5, "10%" = q10, "50%" = q50, "90%" = q90, "97.5%" = q97p5)
}

q1 <- norm_quantiles(fit1)
q2 <- norm_quantiles(fit2)

simg("figures/sim-gauss-01", height = 5.5, width = 5)
plot(d, ylim = range(c(d$y, q1, q2)), pch = 16, col = "lightgray",
  main = expression(paste(y, " ~ N(", mu, "=", beta[0] + beta[1]*x, ", ", log(sigma), "=", beta[0], ")")))
points(d$x, d$y, col = gray(0.4))
plot2d(q1 ~ d$x, add = TRUE, lty = 1)
dev.off()

simg("figures/sim-gauss-02", height = 5.5, width = 5)
plot(d, ylim = range(c(d$y, q1, q2)), pch = 16, col = "lightgray",
  main = expression(paste(y, " ~ N(", mu, "=", beta[0] + beta[1]*x, ", ", log(sigma), "=", beta[0] + beta[1]*x, ")")))
points(d$x, d$y, col = gray(0.4))
plot2d(q2 ~ d$x, add = TRUE, lty = 1)
dev.off()

## Generalized Pareto for modeling extreme precipitation.
load("Pmax.rda")
load("Pmodels.rda")

## Seasonal effects for parameter xi and sigma.
co2 <- unique(Pmax[, c("lon", "lat", "alt")])
fit_s_xi <- fit_s_sigma <- list()
for(i in 1:nrow(co2)) {
  cat("station", i, "of", nrow(co2), "\n")
  d <- data.frame("yday" = 1:365, "lon" = co2[i, 1], "lat" = co2[i, 2], "alt" = co2[i, 3])
  fit_s_xi[[i]] <- predict(b1, newdata = d, model = "xi", term = "yday", intercept = FALSE)
  fit_s_sigma[[i]] <- predict(b1, newdata = d, model = "sigma", term = "yday", intercept = FALSE)
}
fit_s_xi <- do.call("cbind", fit_s_xi)
fit_s_sigma <- do.call("cbind", fit_s_sigma)
m_fit_s_xi <- predict(b1, newdata = data.frame("yday" = 1:365), model = "xi", term = "ti(yday)", intercept = FALSE)
m_fit_s_sigma <- predict(b1, newdata = data.frame("yday" = 1:365), model = "sigma", term = "ti(yday)", intercept = FALSE)

## Altitude and time effects.
ndy <- data.frame("year" = 1936:2016)
fit_y_xi <- predict(b1, newdata = ndy, model = "xi", term = "s(year)", intercept = FALSE, FUN = c95)
fit_y_sigma <- predict(b1, newdata = ndy, model = "sigma", term = "s(year)", intercept = FALSE, FUN = c95)

nda <- data.frame("alt" = seq(min(Pmax$alt), max(Pmax$alt), length = 200))
fit_alt_xi <- predict(b1, newdata = nda, model = "xi", term = "s(alt)", intercept = FALSE, FUN = c95)
fit_alt_sigma <- predict(b1, newdata = nda, model = "sigma", term = "s(alt)", intercept = FALSE, FUN = c95)

## Spatial effects,
## load topographic information and maps of Germany.
Germany <- readRDS("DEU_adm0.rds")
Germany1 <- readRDS("DEU_adm1.rds")
EU <- getMap(resolution = "high")
GermanyTopo <- raster("GermanyTopo.tif")
co <- coordinates(GermanyTopo)
alt <- extract(GermanyTopo, co)
nds <- na.omit(data.frame("alt" = alt, "lon" = co[, "x"], "lat" = co[, "y"]))

fit_sp_xi <- predict(b1, newdata = nds, model = "xi", term = "ti(lon,lat)", intercept = FALSE)
fit_sp_sigma <- predict(b1, newdata = nds, model = "sigma", term = "ti(lon,lat)", intercept = FALSE)

## y-limits for the plots.
ylim_xi <- range(c(fit_s_xi, fit_y_xi, fit_alt_xi, fit_sp_xi))
ylim_sigma <- range(c(fit_s_sigma, fit_y_sigma, fit_alt_sigma, fit_sp_sigma))
ylim_xi <- c(-1, 1) * max(abs(ylim_xi))
ylim_sigma <- c(-1, 1) * max(abs(ylim_sigma))

## Latitude specific colors for the seasonal
## effect plots.
redblue1 <- hcl(c(260, 0, 0), c(80, 0, 80), 30, alpha = c(0.15, 0.1, 0.15))
redblue1[2] <- rainbow_hcl(3, alpha = 0.3)[2]
redblue2 <- hcl(c(260, 0, 0), c(80, 0, 80), 30, alpha = c(0.15, 0.1, 0.15) * 3)
redblue2[2] <- rainbow_hcl(3, alpha = 0.3)[2]
redblue3 <- redblue2
redblue3[2] <- rainbow_hcl(3, alpha = 0.6)[2]
latbreaks <- round(seq(min(co2$lat), max(co2$lat), length = 4), 2)
latcat <- cut(co2$lat, breaks = latbreaks)

simg("figures/pmodel-season-xi", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
matplot(1:365, fit_s_xi, lwd = 1, lty = 1, col = redblue1[latcat], axes = FALSE, xlab = "Time",
  ylab = expression(paste("Effect on ", log(xi))), type = "l", ylim = ylim_xi)
lines(m_fit_s_xi ~ c(1:365), col = "white", lty = 1, lwd = 2.8)
lines(m_fit_s_xi ~ c(1:365), col = "black", lty = 1, lwd = 2)
abline(h = 0, lty = 2, col = rgb(0.1, 0.1, 0.1, alpha = 0.3))
legend("bottomright",
  c(paste(c("North", "Middle", "South"), paste("[", latbreaks[3:1], "-", latbreaks[4:2], " deg N]\t\t\t", sep = "")), "Main effect"),
  col = c(rev(redblue3), "black"), lty = 1, lwd = 2, bty = "n", cex = 0.6)
Axis(as.Date(c("1970-01-01", "1970-12-31")), side = 1)
axis(2)
box()
dev.off()

simg("figures/pmodel-season-sigma", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
matplot(1:365, fit_s_sigma, lwd = 1, lty = 1, col = redblue1[latcat], axes = FALSE, xlab = "Time",
  ylab = expression(paste("Effect on ", log(sigma))), type = "l", ylim = ylim_sigma)
lines(m_fit_s_sigma ~ c(1:365), col = "white", lty = 1, lwd = 2.8)
lines(m_fit_s_sigma ~ c(1:365), col = "black", lty = 1, lwd = 2)
abline(h = 0, lty = 2, col = rgb(0.1, 0.1, 0.1, alpha = 0.3))
legend("bottomright",
  c(paste(c("North", "Middle", "South"), paste("[", latbreaks[3:1], "-", latbreaks[4:2], " deg N]\t\t\t", sep = "")), "Main effect"),
  col = c(rev(redblue3), "black"), lty = 1, lwd = 2, bty = "n", cex = 0.6)
Axis(as.Date(c("1970-01-01", "1970-12-31")), side = 1)
axis(2)
box()
dev.off()

## One dimensional plots.
simg("figures/pmodel-time-xi", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
plot2d(fit_y_xi ~ year, data = ndy, fill.select = c(0, 1, 0, 1), scheme = 2, grid = 200,
  xlab = "Time", ylab = expression(paste("Effect on ", log(xi))), ylim = ylim_xi)
abline(h = 0, lty = 2, col = rgb(0.1, 0.1, 0.1, alpha = 0.3))
dev.off()

simg("figures/pmodel-time-sigma", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
plot2d(fit_y_sigma ~ year, data = ndy, fill.select = c(0, 1, 0, 1), scheme = 2, grid = 200,
  xlab = "Time", ylab = expression(paste("Effect on ", log(sigma))), ylim = ylim_sigma)
abline(h = 0, lty = 2, col = rgb(0.1, 0.1, 0.1, alpha = 0.3))
dev.off()

simg("figures/pmodel-alt-xi", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
plot2d(fit_alt_xi ~ alt, data = nda, fill.select = c(0, 1, 0, 1), scheme = 2, grid = 200,
  xlab = "Altitude", ylab = expression(paste("Effect on ", log(xi))), ylim = ylim_xi)
abline(h = 0, lty = 2, col = rgb(0.1, 0.1, 0.1, alpha = 0.3))
dev.off()

simg("figures/pmodel-alt-sigma", height = 2.65, width = 2.9)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
plot2d(fit_alt_sigma ~ alt, data = nda, fill.select = c(0, 1, 0, 1), scheme = 2, grid = 200,
  xlab = "Altitude", ylab = expression(paste("Effect on ", log(sigma))), ylim = ylim_sigma)
abline(h = 0, lty = 2, col = rgb(0.1, 0.1, 0.1, alpha = 0.3))
dev.off()

## Spatial effects.
simg("figures/pmodel-spatial-xi", type = "png", height = 2.65, width = 2.9, res = 150)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
plotmap(Germany, xlab = "Longitude [deg]", ylab = "Latitude [deg]", lwd = 0.3)
plot(EU, add = TRUE, lwd = 0.3)
bamlss:::xymap(lon, lat, fit_sp_xi, data = nds, pos = "bottomright", layout = FALSE, map = FALSE,
  add = TRUE, color = diverge_hcl, shift = c(0.15, 0.06), distance.labels = 0,
  symmetric = TRUE, swap = FALSE, digits = 2,
  side.legend = 2, side.ticks = 2, cex.labels = 0.7, range = ylim_xi)
plot(Germany, add = TRUE, lwd = 0.3)
plot(Germany1, add = TRUE, lwd = 0.3)
text(14, 49.97, expression(log(xi)), cex = 0.8, pos = 4)
box()
axis(1)
axis(2)
box()
dev.off()

simg("figures/pmodel-spatial-sigma", type = "png", height = 2.65, width = 2.9, res = 150)
par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
plotmap(Germany, xlab = "Longitude [deg]", ylab = "Latitude [deg]", lwd = 0.3)
plot(EU, add = TRUE, lwd = 0.3)
bamlss:::xymap(lon, lat, fit_sp_sigma, data = nds, pos = "bottomright", layout = FALSE, map = FALSE,
  add = TRUE, color = diverge_hcl, shift = c(0.15, 0.06), distance.labels = 0,
  symmetric = TRUE, swap = FALSE, digits = 2,
  side.legend = 2, side.ticks = 2, cex.labels = 0.7, range = ylim_sigma)
plot(Germany, add = TRUE, lwd = 0.3)
plot(Germany1, add = TRUE, lwd = 0.3)
text(14, 49.97, expression(log(sigma)), cex = 0.8, pos = 4)
box()
axis(1)
axis(2)
box()
dev.off()

## Calculate 24-hour 100 years return levels.
## First, compute exceedence rates from thresholds.
load("thresholds.rda")
Lambda <- list()
for(i in levels(thresholds$id)) {
  d <- subset(WeatherGermany, id == i)
  freq <- sum(d$rain >= thresholds$threshold[thresholds$id == i]) / nrow(d)
  Lambda[[i]] <- data.frame("frequency" = freq, "lon" = d$lon[1], "lat" = d$lat[1], "alt" = d$alt[1])
}
Lambda <- do.call("rbind", Lambda)
Lambda <- subset(Lambda, alt <= 1500)

coordinates(Lambda) <- c("lon", "lat")
co <- coordinates(GermanyTopo)
alt <- extract(GermanyTopo, co)

Ggrid <- data.frame("lon" = co[, 1], "lat" = co[, 2], "alt" = alt)

gridded(Ggrid) <- ~ lon + lat
projection(Ggrid) <- proj4string(GermanyTopo)
projection(Lambda) <- proj4string(GermanyTopo)

dh <- spTransform(Lambda, CRS("+init=epsg:32610"))
dg <- spTransform(Ggrid, CRS("+init=epsg:32610"))

b <- autoKrige(frequency ~ 1 + alt, dh, dg)

br <- data.frame("fit" = b$krige_output$var1.pred, "lon" = co[, 1], "lat" = co[, 2])
coordinates(br) <- ~ lon + lat
gridded(br) <- TRUE
projection(br) <- proj4string(GermanyTopo)
br <- raster(br)

writeRaster(br, filename = "GermanyExceedFrequency.tif", overwrite = TRUE)

## Predict return levels for each day of the year.
if(!file.exists("r100.rda")) {
  load("Pmodels.rda")
  thresvals <- raster("GermanyThresholds.tif")
  exvals <- raster("GermanyExceedFrequency.tif")
  r100 <- matrix(0, nrow = nrow(nds), ncol = 365)
  thresvals <- as.numeric(extract(thresvals, as.matrix(nds[, c("lon", "lat")])))
  exvals <- as.numeric(extract(exvals, as.matrix(nds[, c("lon", "lat")])))
  for(i in 1:365) {
    cat("day", i, "\n")
    nds$yday <- i
    p <- predict(b1, newdata = nds, term = c("alt", "yday", "lon", "lat"), cores = 4)
    r100[, i] <- thresvals + exp(p$sigma) / exp(p$xi) * ((365 * 100 * exvals)^(exp(p$xi)) - 1)
  }
  save(r100, file = "r100.rda")
} else {
  load("r100.rda")
}

## Calculate maximum 100 years return levels for
## each month of the year for each location, i.e.,
## the maximum return level within each month.
i <- sort(rep(1:12, length.out = 365))
i <- split(1:365, factor(i))

r100v <- list()
for(j in 1:length(i))
  r100v[[j]] <- apply(r100[, i[[j]]], 1, max)
lr <- range(unlist(r100v))
rr <- quantile(unlist(r100v), prob = c(0.05, 0.99))

## Function for the color palette.
pal <- function (n, h = c(-100, 100), c. = c(60, 100), l = c(15, 95), 
    power = c(2, 2.14772727272727), fixup = TRUE, gamma = NULL, 
    alpha = 1, ...) 
{
    if (!is.null(gamma)) 
        warning("'gamma' is deprecated and has no effect")
    if (n < 1L) 
        return(character(0L))
    h <- rep(h, length.out = 2L)
    c <- rep(c., length.out = 2L)
    l <- rep(l, length.out = 2L)
    power <- rep(power, length.out = 2L)
    rval <- seq(1, 0, length = n)
    rval <- hex(polarLUV(L = l[2L] - diff(l) * rval^power[2L], 
        C = c[2L] - diff(c) * rval^power[1L], H = h[2L] - diff(h) * 
            rval), fixup = fixup, ...)
    if (!missing(alpha)) {
        alpha <- pmax(pmin(alpha, 1), 0)
        alpha <- format(as.hexmode(round(alpha * 255 + 1e-04)), 
            width = 2L, upper.case = TRUE)
        rval <- paste(rval, alpha, sep = "")
    }
    return(rval)
}

## Now, cycle through all months and plot.
for(j in 1:length(i)) {
  simg(paste0("figures/pmodel-r100-", month.name[j]),
    type = "png", height = 2.65, width = 2.9, res = 150)
  par(mar = c(4.1, 4.1, 1.1, 1.1), cex = 0.7)
  plotmap(Germany, xlab = "Longitude [deg]", ylab = "Latitude [deg]", lwd = 0.3)
  plot(EU, add = TRUE, lwd = 0.3)
  bamlss:::xymap(lon, lat, r100v[[j]], data = nds, pos = "bottomright", layout = FALSE, map = FALSE,
    add = TRUE, color = pal, shift = c(0.15, 0.06), distance.labels = 0,
    symmetric = FALSE, swap = FALSE, digits = 0,
    side.legend = 2, side.ticks = 2, cex.labels = 0.7)
  plot(Germany, add = TRUE, lwd = 0.3)
  plot(Germany1, add = TRUE, lwd = 0.3)
  text(13, 50, "Precip. [mm/24h]", cex = 0.7, pos = 4)
  box()
  axis(1)
  axis(2)
  text(3, 54.8, month.name[j], pos = 4)
  box()
  dev.off()
}

## Spatial distribution of meteorological stations.
at <- seq(0, 1500, by = 500)
pal <- colorlegend(x = co$alt, plot = FALSE, color = terrain_hcl,
  range = c(0, 1500), ncol = 6)

simg("figures/pmodel-stations", type = "png", width = 4.49, height = 4.5)
par(mar = c(4.1, 4.1, 1.5, 1.5))
plot(Germany, col = rgb(0, 0, 1, alpha = 0.2), lwd = 0.6, ylim = c(min(co[, 2]) - 0.5, max(co[, 2]) + 1))
points(co[, 1], co[, 2], pch = 21, col = "black", bg = pal$map(co$alt), cex = 0.7)
box()
axis(1)
axis(2)
mtext("Longitude [deg]", side = 1, line = 3)
mtext("Latitude [deg]", side = 2, line = 3)
legend("topleft", "Meteorological station", pch = 21,
  col = "black",
  pt.bg = "black",
  box.col = NA, bg = NA, cex = 0.9)
colorlegend(x = co$alt, plot = FALSE, color = terrain_hcl, add = TRUE,
  pos = "bottomright", side.legend = 2, side.ticks = 2,
  shift = c(0.18, 0.06), distance.labels = -0.2, swap = FALSE, digits = 0,
  symmetric = FALSE, title = "Altitude",
  range = c(0, 1500), ncol = 6, at = at, cex.labels = 0.9,
  labels = c("0", "500", "1000", "1500+"))
text(13.95, 50.2, "Altitude [m]", cex = 0.9, pos = 4)
dev.off()

## Histogram and density from fitted GP model.
b0 <- bamlss(rain2 ~ 1, data = Pmax, family = "gpareto", sampler = FALSE, binning = TRUE, nu = 1)
f <- family(b0)
dens <- f$d(Pmax$rain2, predict(b0, type = "parameter"))

simg("figures/pmodel-hist", width = 4.49, height = 4.5)
par(mar = c(4.1, 4.1, 1.5, 1.5))
hist(Pmax$rain2, freq = FALSE, main = "", col = gray(0.9),
  xlab = "Precipitation [mm/24h]", ylab = "Density",
  breaks = "Scott")
orange <- rgb(242, 146, 0, maxColorValue = 255)
f <- family(b0)
lines(dens[order(Pmax$rain2)] ~ Pmax$rain2[order(Pmax$rain2)], col = orange, lwd = 1.5)
legend("right", "Fitted GP distribution", lwd = 2, col = orange,
  box.col = NA, bg = NA, cex = 0.95)
box()
dev.off()

## Interpretation of the parameters of the GP distribution.
simg("figures/pmodel-density-example", width = 4.49, height = 4.5)
par <- list("xi" = c(0.1, 0.1, 0.9, 0.9), "sigma" = c(5, 10, 5, 10))
y <- seq(0, 60, length = 100)
f <- gpareto_bamlss()
d1 <- f$d(y, list("xi" = par$xi[1], "sigma" = par$sigma[1]))
d2 <- f$d(y, list("xi" = par$xi[2], "sigma" = par$sigma[2]))
d3 <- f$d(y, list("xi" = par$xi[3], "sigma" = par$sigma[3]))
d4 <- f$d(y, list("xi" = par$xi[4], "sigma" = par$sigma[4]))
col <- rainbow_hcl(4)
matplot(y, cbind(d1, d2, d3, d4), type = "l", lwd = 2, lty = 1, col = col,
  xlab = "y", ylab = "Density")
ex <- c(
  expression(paste(xi == 0.1, ", ", sigma == 5)),
  expression(paste(xi == 0.1, ", ", sigma == 10)),
  expression(paste(xi == 0.9, ", ", sigma == 5)),
  expression(paste(xi == 0.9, ", ", sigma == 10))
)
legend("topright", ex,
  lwd = 2, col = col, box.col = NA, bg = NA)
dev.off()

