###################################
## German extreme precipitation. ##
###################################
## The second example in the manuscript focuses on
## extreme precipitation events in Germany. The data at
## hand are 24h precipitation records measured using
## a quite dense net of meteorological stations.
## More specifically, we are interested in the 100 year
## return levels of precipitation, i.e., daily maximum
## precipitation records that may occur at least once in 100 years.
## We use a generalized Pareto distribution for this task,
## as this is a common way of modeling only the right-hand or
## left-hand side of distributions and is known to have superior
## predictive performance for extreme values than modeling
## the whole distribution, e.g., using a censored Gaussian model.

## Using peak-over-thresholds models in combination with the
## generalized Pareto distribution, we first need to compute
## suitable thresholds for each meteorological station. Finding
## the thresholds is done using a grid search of the range of
## 24h precipitation records. To find the most suitable threshold
## we estimate a full generalized Pareto model for each station
## and for each threshold, beginning with the lowest. The models
## consist of a full year effect and a seasonal effect.
## After estimation we look at the corresponding quantile residuals.
## If the residuals behave seem to be normal distributed according
## to a simple t-test and a Shapiro-Wilk test, we save the threshold.

## Required packages.
library("bamlss")
library("raster")
library("sp")
library("automap")

## Compile the threshold corrected data set.
if(!file.exists("Pmax.rda")) {
  load("WeatherGermany.rda")

  ## Find thresholds for each station.
  if(!file.exists("thresholds.rda")) {
    thresholds <- NULL
    stations <- unique(WeatherGermany$id)
    ## Cycle over all stations.
    for(i in seq_along(stations)) {
      cat("station:", i, "of", length(stations), "\n")
      d <- subset(WeatherGermany, id == stations[i] & rain > 0)
      if(all(d$rain < 0) | nrow(d) < 365)
        next

      ## Setup the grid of thresholds.
      tv <- seq(min(d$rain, na.rm = TRUE), quantile(d$rain, prob = 0.95, na.rm = TRUE), length = 100)
      tv <- tv[tv > 0]
      save_tv <- NULL

      ## Cycle through the thresholds starting with the lowest.
      for(j in seq_along(tv)) {
        cat("... threshold:", tv[j], "\n")

        ## Subset the data.
        Pmax <- subset(d, rain >= tv[j])
        Pmax$rain2 <- Pmax$rain - min(Pmax$rain)

        ## Setup a model formula.
        f <- list(
          rain2 ~ s(year) + s(yday,bs="cc"),
          sigma ~ s(year) + s(yday,bs="cc")
        )

        ## Estimate model.
        b0 <- try(bamlss(f, data = Pmax, family = "gpareto", sampler = FALSE, verbose = FALSE, binning = TRUE))
        if(inherits(b0, "try-error"))
          next

        ## Extract the quantile residuals.
        e0 <- residuals(b0)
        e <- e0[is.finite(e0)]

        ## Check for normality.
        pval <- t.test(e)$p.value
        if(pval > 0.05) {
          if(length(e) > 5000)
            e <- e[sample(1:length(e), size = 5000)]
          save_tv <- rbind(save_tv,
            c("threshold" = tv[j], "p.value" = shapiro.test(e)$p.value))
        }
        if(!is.null(save_tv)) {
          if(length(pvals <- save_tv[, "p.value"]) > 2) {
            if(pvals[length(pvals)] <= pvals[length(pvals) - 1]) {
              plot(e0)
              break
            }
          }
        }
      }

      ## Save the best threshold for station i according to
      ## the maximum p-value of the Shapiro-Wilk test.
      if(!is.null(save_tv)) {
        final_tv <- save_tv[which.max(save_tv[, "p.value"]), "threshold"]
        thresholds <- rbind(thresholds, data.frame("id" = stations[i], "threshold" = final_tv,
          "lon" = unique(d$lon), "lat" = unique(d$lat)))
      } else warning(paste0("could not find suitable threshold for station ", stations[i], "!"))
    }
    save(thresholds, file = "thresholds.rda")

    ## For predicting the final 100 year return levels, we need
    ## to compute a grid of thresholds also for locations without
    ## observations. Therefore, we use universal Kriging to estimate
    ## the threshold surface.
    thresholds$alt <- NA
    for(i in levels(thresholds$id)) {
      alt <- unique(WeatherGermany$alt[WeatherGermany$id == i])
      thresholds$alt[thresholds$id == i] <- alt
    }

    thresholds <- subset(thresholds, alt <= 1500)

    coordinates(thresholds) <- c("lon", "lat")

    GermanyTopo <- raster("GermanyTopo.tif")
    co <- coordinates(GermanyTopo)
    alt <- extract(GermanyTopo, co)

    Ggrid <- data.frame("lon" = co[, 1], "lat" = co[, 2], "alt" = alt)

    gridded(Ggrid) <- ~ lon + lat
    projection(Ggrid) <- proj4string(GermanyTopo)
    projection(thresholds) <- proj4string(GermanyTopo)

    dh <- spTransform(thresholds, CRS("+init=epsg:32610"))
    dg <- spTransform(Ggrid, CRS("+init=epsg:32610"))

    ## Estimate the Kriging model.
    b <- autoKrige(threshold ~ 1 + alt, dh, dg)

    ## Write the predictions to a "raster" file on disk.
    br <- data.frame("fit" = b$krige_output$var1.pred, "lon" = co[, 1], "lat" = co[, 2])
    coordinates(br) <- ~ lon + lat
    gridded(br) <- TRUE
    projection(br) <- proj4string(GermanyTopo)
    br <- raster(br)

    writeRaster(br, filename = "dGermanyThresholds.tif", overwrite = TRUE)
  } else {
    load("thresholds.rda")
  }

  ## Finally, compile the precipitation data set using the
  ## thresholds.
  Pmax <- list()
  for(i in levels(thresholds$id)) {
    d <- subset(WeatherGermany, id == i)
    d <- subset(d, rain >= thresholds$threshold[thresholds$id == i])
    d$rain2 <- d$rain - min(d$rain)
    Pmax[[i]] <- d
  }
  Pmax <- do.call("rbind", Pmax)
  Pmax$id <- as.factor(Pmax$id)

  save(Pmax, file = "Pmax.rda")
} else {
  load("Pmax.rda")
}

## Generalized Pareto models.
if(!file.exists("Pmodels.rda")) {
  load("Pmax.rda")

  ## Start with an intercept only model.
  ## Note the warnings are not critical, they may result from using
  ## half-stepping during parameter updating, which is controlled by setting e.g. nu = 1.
  b0 <- bamlss(rain2 ~ 1, data = Pmax, family = "gpareto", sampler = FALSE, binning = TRUE, nu = 1)

  ## Model formula for the complex space-time model.
  f <- list(
    rain2 ~ s(alt,k=10,bs="tp") + s(year,bs="tp",k=10) + ti(yday,bs="cp",k=15) + ti(lon,lat,k=100,d=2,bs="tp") +
      ti(yday,lon,lat,bs=c("cp","tp"),d=c(1,2),k=c(10,30)),
    sigma ~ s(alt,k=10,bs="tp") + s(year,bs="tp",k=10) + ti(yday,bs="cp",k=15) + ti(lon,lat,k=100,d=2,bs="tp") +
      ti(yday,lon,lat,bs=c("cp","tp"),d=c(1,2),k=c(10,30))
  )

  ## Use posterior mode estimation to have a first look at the effects.
  b1 <- bamlss(f, data = Pmax, family = "gpareto", nu = 1,
    start = coef(b0), sampler = FALSE, binning = TRUE)

  ## Now, use the coefficients of model b1 as starting values for
  ## a full MCMC model.
  set.seed(123)
  b2 <- bamlss(f, data = Pmax, family = "gpareto", start = coef(b1),
    n.iter = 12000, burnin = 2000, thin = 10, optimizer = FALSE, binning = TRUE)

  save(b0, b2, file = "Pmodels.rda")
}

