########################################
## Daily temperature at the Zugspitze ##
########################################
## This is the first example of the manuscript modeling
## daily temperature observations of the highest mountain
## in Germany, the Zugspitze. The aim of this analysis is
## to estimate a time-trend and investigate if temperature
## observations have significantly increased over the years.

## Required packages.
library("bamlss")
library("gamlss")

## Load the data.
load("Tdata.rda")

## Compute lag-difference variable, this is needed
## because of autocorrelation in the data. We compute
## the difference and not use a lag operator because
## we are interested in the time trend and seasonal
## trend. When using a lag operator this information
## will be captured by the lag and therefore estimated
## time trends and seasonal trends are underestimated.
Tdata$T_diff <- Tdata$T - Tdata$T_lag

## Start with a simpel Gaussian model and quite low
## number of MCMC iterations and no burn-in pahse. The
## resulting sampling paths of the models are illustrated
## in Figure 4 in the manuscript.
f <- list(
  T ~ s(T_diff) + ti(yday,bs="cc") + ti(year) + ti(yday,year,bs=c("cc","cr"),k=c(10,10)),
  sigma ~ s(T_diff) + ti(yday,bs="cc") + ti(year) + ti(yday,year,bs=c("cc","cr"),k=c(10,10))
)

if(!file.exists("Tmodel-b0.rda")) {
  set.seed(123)
  b0 <- bamlss(f, data = Tdata, family = "gaussian",
    optimizer = FALSE, n.iter = 1000, burnin = 0)
  save(b0, file = "Tmodel-b0.rda")
}

## Increase the burn-in phase, since samples did not
## converge and have high auto-correlation.
if(!file.exists("Tmodel-b1.rda")) {
  set.seed(123)
  b1 <- bamlss(f, data = Tdata, family = "gaussian",
    optimizer = bfit, n.iter = 1200, burnin = 200, thin = 1)
  save(b1, file = "Tmodel-b1.rda")
}

## Still high autocorrelation, therfore, we increase again
## the number of MCMC iterations, as well as the burn-in phase.
if(!file.exists("Tmodel-b2.rda")) {
  set.seed(123)
  b2 <- bamlss(f, data = Tdata, family = "gaussian",
    optimizer = bfit, n.iter = 6000, burnin = 2000, thin = 10,
    cores = 8, combine = FALSE)
  save(b2, file = "Tmodel-b2.rda")
}

## Looking at the quantile residuals in Figure 5 suggests that
## the Gaussian model might not be the best choice. Therefore,
## we use the 4 parameytric Box-Cox t-distribution.
f <- list(
  T2 ~ s(T_diff) + ti(yday,bs="cc") + ti(year) + ti(yday,year,bs=c("cc","cr"),k=c(10,10)),
  sigma ~ s(T_diff) + ti(yday,bs="cc") + ti(year) + ti(yday,year,bs=c("cc","cr"),k=c(10,10)),
  nu ~ s(T_diff) + ti(yday,bs="cc") + ti(year) + ti(yday,year,bs=c("cc","cr"),k=c(10,10)),
  tau ~ 1
)

if(!file.exists("Tmodel-b3.rda")) {
  ## The BCT distribution is defined on (0, Inf), therefore, we shift
  ## the temperature observations to ensure positiveness.
  shift <- abs(min(Tdata$T)) + 1
  Tdata$T2 <- Tdata$T + shift

  set.seed(123)
  b3 <- bamlss(f, data = Tdata, family = BCT(mu.link = "log"),
    optimizer = bfit, n.iter = 6000, burnin = 2000, thin = 10,
    cores = 8, combine = FALSE)

  save(b3, file = "Tmodel-b3.rda")
}

## As a final setup, we use the 4 parametric Box-Cox Power Exponential distribution.
if(!file.exists("Tmodel-b4.rda")) {
  ## The BCPE distribution is defined on (0, Inf), therefore, we shift
  ## the temperature observations to ensure positiveness.
  shift <- abs(min(Tdata$T)) + 1
  Tdata$T2 <- Tdata$T + shift

  set.seed(123)
  b4 <- bamlss(f, data = Tdata, family = BCPE(mu.link = "log"),
    optimizer = bfit, n.iter = 6000, burnin = 2000, thin = 10,
    cores = 8, combine = FALSE, nu = 1)

  save(b4, file = "Tmodel-b4.rda")
}

